==> prepare-raspbian-sd-v1.3.1.tar.gz: prepare-raspbian-sd/bin/prepare-raspbian-config-json-api.sh <==
#!/bin/bash -e
#
# prepare-raspbian-sd:bin/prepare-raspbian-config-json-api.sh
#
declare -gr __PREPARE_RASPBIAN_CONFIG_DIR__="${HOME}/.config/prepare-raspbian-sd"
declare -gr _PREPARE_RASPBIAN_CONFIG_FILE_="${__PREPARE_RASPBIAN_CONFIG_DIR__}/config.json"
declare -gr _PREPARE_RASPBIAN_CONFIG_VARS_=(
'DEFAULT_SD_DEV_NAME:STRING'
'GPU_MEM:INTEGER'
'RASPBIAN_DEFAULT_BACKUP_DIRECTORY:STRING'
'RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY:STRING'
'RASPBIAN_DEFAULT_IMG:STRING'
'RASPBIAN_PI_GROUP_ID:INTEGER'
'RASPBIAN_PI_USER_ID:INTEGER'
'SSH_AUTHORIZED_KEYS:STRING'
)

function prepare_raspbian_show_list {
  local entry=

  for entry in "${_PREPARE_RASPBIAN_CONFIG_VARS_[@]}" ; do
    cut -d ':' -f1 <<<"${entry}"
  done
}

function prepare_raspbian_get_type {
  local -r var_name="$1"
  local    entry=

  for entry in "${_PREPARE_RASPBIAN_CONFIG_VARS_[@]}" ; do
    if [ "${entry:0:${#var_name} + 1}" = "${var_name}:" ] ; then
      cut -d ':' -f2 <<<"${entry}"
    fi
  done
}

function prepare_raspbian_show_list_all_DEPRECATED {
  local var=

  while read -r var ; do
    prepare_raspbian_check_and_get_value "${var}"
  done < <( prepare_raspbian_show_list )
}

function prepare_raspbian_show_list_all {
  ensure_raspbian_config_ready

  jq -S '.' "${_PREPARE_RASPBIAN_CONFIG_FILE_}" || exit 1
}

function prepare_raspbian_config_is_var_name_valide {
  local -r var_name="$1"
  local    var=

  while read -r var ; do
   if [ "${var_name}" = "${var}" ] ; then
    return 0
   fi
  done < <( prepare_raspbian_show_list )

  return 1
}

function prepare_raspbian_config_check_var_name {
  local -r var_name="$1"

  if ! prepare_raspbian_config_is_var_name_valide "${var_name}" ; then
    echofatal "Wrong config entry. What is '${var_name}' ?"
    return 1
  fi

  return 0
}

function ensure_raspbian_config_ready {
  if [ ! -d "${__PREPARE_RASPBIAN_CONFIG_DIR__}" ] ; then
    mkdir -vp "${__PREPARE_RASPBIAN_CONFIG_DIR__}"
  fi

  exit_if_directory_not_exists "${__PREPARE_RASPBIAN_CONFIG_DIR__}"

  touch "${_PREPARE_RASPBIAN_CONFIG_FILE_}"
  exit_if_file_not_exists "${_PREPARE_RASPBIAN_CONFIG_FILE_}"
}

function prepare_raspbian_initialise_if_needed {
  local -r var_name="$1"
  local -r default_value="${2:-}"

  ensure_raspbian_config_ready

  if ! is_variable_define_and_not_empty_safe "${var_name}" ; then
    local value=
    local json_config_for_var=

    json_config_for_var="$( jq ".[\"${var_name}\"]" <"${_PREPARE_RASPBIAN_CONFIG_FILE_}" )" || exit 1

    if [ "${json_config_for_var}" != 'null' ] ; then
      value="$( jq -r '.value' <<<"${json_config_for_var}" )" || exit 1

      if [ "${value}" = 'null' ] ; then
        local link=
        link="$( jq -r '.link' <<<"${json_config_for_var}" )" || exit 1

        if [ "${link}" != 'null' ] ; then
          if [ -f "${link}" ] ; then
            value="$( <"${link}" )"
          else
            echofatal "'${var_name}' link not valid: '${link}'"
            exit 1
          fi
        else
          value=
        fi
      fi
    fi

    declare -g "${var_name}"="${value}"

    if ! is_variable_define_and_not_empty_safe "${var_name}" ; then
      if [ -n "${default_value}" ] ; then
        echowarn "${var_name} is not define" >&2
        declare "${var_name}"="${default_value}"
        echowarn "Use ${var_name}=${!var_name}" >&2
      fi
    fi

    if is_variable_define_and_not_empty_safe "${var_name}" ; then
      declare -gr "${var_name}=${!var_name}"
      # shellcheck disable=SC2163
      export "${var_name}"
    fi
  fi
}

function _prepare_raspbian_config_json_set_ {
  local -r var_name="$1"
  local -r vat_name_type="$2"
  local -r var_value="$3"
  local    config=

  ensure_raspbian_config_ready

  config="$(
      jq -S ". | del(.[\"${var_name}\"] | .value) | del(.[\"${var_name}\"] | .link )" \
        "${_PREPARE_RASPBIAN_CONFIG_FILE_}"
      )" || exit 1

  if [ -n "${var_value}" ] ; then
    local var_value_json=

    if [ "${vat_name_type}" != 'link' ] ; then
      local var_value_type=

      var_value_type="$( prepare_raspbian_get_type "${var_name}" )" || exit 1

      case "${var_value_type}" in
        'INTEGER')
          var_value_json="${var_value}"
          ;;

        *)
         var_value_json="\"${var_value}\""
          ;;
      esac
    else
      var_value_json="\"${var_value}\""
    fi

    config="$(
      jq -S ". |
        (.[\"${var_name}\"] | .${vat_name_type} ) |= ${var_value_json}" <<<"${config}"
      )" || exit 1

    echo "${config}" >"${_PREPARE_RASPBIAN_CONFIG_FILE_}" || exit 1
    echoinfo "'${var_name}'.'${vat_name_type}' is set : '${var_value}'" >&2
  else
    echo "${config}" >"${_PREPARE_RASPBIAN_CONFIG_FILE_}" || exit 1
    echoinfo "'${var_name}' is removed." >&2
  fi
}

function _prepare_raspbian_config_set_ {
  local -r var_name="$1"
  local -r var_value="$2"

  _prepare_raspbian_config_json_set_ "${var_name}" 'value' "${var_value}"
}

function prepare_raspbian_check_and_set_value_string {
  local -r var_name="$1"
  local -r var_value="$2"

  if prepare_raspbian_config_check_var_name "${var_name}" ; then
    _prepare_raspbian_config_set_ "${var_name}" "${var_value}"
  else
    return 1
  fi
}

function prepare_raspbian_check_and_set_value_integer {
  local -r var_name="$1"
  local -r var_value="$2"

  if prepare_raspbian_config_check_var_name "${var_name}" ; then
    _prepare_raspbian_config_set_ "${var_name}" "${var_value}"
  else
    return 1
  fi
}

function _prepare_raspbian_config_set_link_ {
  local -r var_name="$1"
  local -r var_link="$2"

  ensure_raspbian_config_ready

  _prepare_raspbian_config_json_set_ "${var_name}" 'link' "${var_link}"
}

function prepare_raspbian_check_and_set_value_link {
  local -r var_name="$1"
  local -r var_link="$2"

  if prepare_raspbian_config_check_var_name "${var_name}" ; then
    _prepare_raspbian_config_set_link_ "${var_name}" "${var_link}"
  else
    return 1
  fi
}

function prepare_raspbian_check_and_set_value {
  local -r var_type="$1"
  local -r var_name="$2"
  local -r var_value="$3"

  case "${var_type}" in
    'INTEGER')
      if ! is_positive_integer "${var_value}" ; then
        echofatal "Value must be a postive integer: '${var_value}'"
        exit 1
      fi
      prepare_raspbian_check_and_set_value_integer "${var_name}" "${var_value}"
      ;;

    'LINK')
      if [ ! -f "${var_value}" ] ; then
        echofatal "File not found: '${var_value}'"
        exit 1
      fi
      prepare_raspbian_check_and_set_value_link "${var_name}" "${var_value}"
      ;;

    'STRING')
      prepare_raspbian_check_and_set_value_string "${var_name}" "${var_value}"
      ;;

    *)
      echofatal "Unsupported variable type: '${var_type}'"
      exit 1
      ;;
  esac
}

function prepare_raspbian_check_and_get_value {
  local -r var_name="$1"

  if prepare_raspbian_config_check_var_name "${var_name}" ; then
    prepare_raspbian_initialise_if_needed "${var_name}"

    if is_variable_define_and_not_empty_safe "${var_name}" ; then
      echo "${var_name}=${!var_name}" >&2
      echo "${!var_name}"
      return 0
    else
      echo "${var_name} not define." >&2
      return 1
    fi
  else
    return 1
  fi
}

==> prepare-raspbian-sd-v1.3.2.tar.gz: prepare-raspbian-sd/bin/prepare-raspbian-config-json-api.sh <==
#!/bin/bash -e
#
# prepare-raspbian-sd:bin/prepare-raspbian-config-json-api.sh
#
declare -gr __PREPARE_RASPBIAN_CONFIG_DIR__="${HOME}/.config/prepare-raspbian-sd"
declare -gr _PREPARE_RASPBIAN_CONFIG_FILE_="${__PREPARE_RASPBIAN_CONFIG_DIR__}/config.json"
declare -gr _PREPARE_RASPBIAN_CONFIG_VARS_=(
'BCM_WORKSPACE:BOOLEAN'
'DEFAULT_SD_DEV_NAME:STRING'
'GPU_MEM:INTEGER'
'RASPBIAN_DEFAULT_BACKUP_DIRECTORY:STRING'
'RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY:STRING'
'RASPBIAN_DEFAULT_IMG:STRING'
'RASPBIAN_PI_GROUP_ID:INTEGER'
'RASPBIAN_PI_USER_ID:INTEGER'
'SSH_AUTHORIZED_KEYS:STRING'
)

function prepare_raspbian_show_list {
  local entry=

  for entry in "${_PREPARE_RASPBIAN_CONFIG_VARS_[@]}" ; do
    cut -d ':' -f1 <<<"${entry}"
  done
}

function __prepare_raspbian_show_list_json__ {
  local first='true'

  echo -n '{ "varname": ['
  while read -r var ; do
    if [ "${first}" = 'true' ] ; then
      first='false'
    else
      echo -n ','
    fi
    echo -n "\"${var}\""
  done < <( prepare_raspbian_show_list )
  echo '] }'
}

function prepare_raspbian_show_list_json {
  __prepare_raspbian_show_list_json__ | jq -S '.' || return $?
}

function prepare_raspbian_get_type {
  local -r var_name="$1"
  local    entry=

  for entry in "${_PREPARE_RASPBIAN_CONFIG_VARS_[@]}" ; do
    if [ "${entry:0:${#var_name} + 1}" = "${var_name}:" ] ; then
      cut -d ':' -f2 <<<"${entry}"
    fi
  done
}

function prepare_raspbian_show_list_all {
  ensure_raspbian_config_ready

  jq -S '.' "${_PREPARE_RASPBIAN_CONFIG_FILE_}" || exit 1
}

function prepare_raspbian_config_is_var_name_valide {
  local -r var_name="$1"
  local    var=

  while read -r var ; do
   if [ "${var_name}" = "${var}" ] ; then
    return 0
   fi
  done < <( prepare_raspbian_show_list )

  return 1
}

function prepare_raspbian_config_check_var_name {
  local -r var_name="$1"

  if ! prepare_raspbian_config_is_var_name_valide "${var_name}" ; then
    echofatal "Wrong config entry. What is '${var_name}' ?"
    return 1
  fi

  return 0
}

function ensure_raspbian_config_ready {
  if [ ! -d "${__PREPARE_RASPBIAN_CONFIG_DIR__}" ] ; then
    mkdir -vp "${__PREPARE_RASPBIAN_CONFIG_DIR__}"
  fi

  exit_if_directory_not_exists "${__PREPARE_RASPBIAN_CONFIG_DIR__}"

  touch "${_PREPARE_RASPBIAN_CONFIG_FILE_}"
  exit_if_file_not_exists "${_PREPARE_RASPBIAN_CONFIG_FILE_}"
}

function prepare_raspbian_initialise_if_needed {
  local -r var_name="$1"
  local -r default_value="${2:-}"

  ensure_raspbian_config_ready

  if ! is_variable_define_and_not_empty_safe "${var_name}" ; then
    local value=
    local json_config_for_var=

    json_config_for_var="$( jq ".[\"${var_name}\"]" <"${_PREPARE_RASPBIAN_CONFIG_FILE_}" )" || exit 1

    if [ "${json_config_for_var}" != 'null' ] ; then
      value="$( jq -r '.value' <<<"${json_config_for_var}" )" || exit 1

      if [ "${value}" = 'null' ] ; then
        local link=
        link="$( jq -r '.link' <<<"${json_config_for_var}" )" || exit 1

        if [ "${link}" != 'null' ] ; then
          if [ -f "${link}" ] ; then
            value="$( <"${link}" )"
          else
            echofatal "'${var_name}' link not valid: '${link}'"
            exit 1
          fi
        else
          value=
        fi
      fi
    fi

    declare -g "${var_name}"="${value}"

    if ! is_variable_define_and_not_empty_safe "${var_name}" ; then
      if [ -n "${default_value}" ] ; then
        echowarn "${var_name} is not define" >&2
        declare "${var_name}"="${default_value}"
        echowarn "Use ${var_name}=${!var_name}" >&2
      fi
    fi

    if is_variable_define_and_not_empty_safe "${var_name}" ; then
      declare -gr "${var_name}=${!var_name}"
      # shellcheck disable=SC2163
      export "${var_name}"
    fi
  fi
}

function _prepare_raspbian_config_json_set_ {
  local -r var_name="$1"
  local -r var_name_type="$2"
  local -r var_value="$3"
  local    config=

  ensure_raspbian_config_ready

  config="$(
      jq -S ". | del(.[\"${var_name}\"] | .value) | del(.[\"${var_name}\"] | .link )" \
        "${_PREPARE_RASPBIAN_CONFIG_FILE_}"
      )" || exit 1

  if [ -n "${var_value}" ] ; then
    local var_value_json=

    if [ "${var_name_type}" != 'link' ] ; then
      local var_value_expected_type=
      var_value_expected_type="$( prepare_raspbian_get_type "${var_name}" )" || exit 1

      case "${var_value_expected_type}" in
        'BOOLEAN'|'INTEGER')
          var_value_json="${var_value}"
          ;;

        *)
         var_value_json="\"${var_value}\""
          ;;
      esac
    else
      var_value_json="\"${var_value}\""
    fi

    config="$(
      jq -S ". |
        (.[\"${var_name}\"] | .${var_name_type} ) |= ${var_value_json}" <<<"${config}"
      )" || exit 1

    echo "${config}" >"${_PREPARE_RASPBIAN_CONFIG_FILE_}" || exit 1
    echoinfo "'${var_name}'.'${var_name_type}' is set : '${var_value}'" >&2
  else
    echo "${config}" >"${_PREPARE_RASPBIAN_CONFIG_FILE_}" || exit 1
    echoinfo "'${var_name}' is removed." >&2
  fi
}

function _prepare_raspbian_config_set_value_ {
  local -r var_name="$1"
  local -r var_value="$2"

  _prepare_raspbian_config_json_set_ "${var_name}" 'value' "${var_value}"
}

function prepare_raspbian_check_and_set_value_string {
  local -r var_name="$1"
  local -r var_value="$2"

  if prepare_raspbian_config_check_var_name "${var_name}" ; then
    _prepare_raspbian_config_set_value_ "${var_name}" "${var_value}"
  else
    return 1
  fi
}

function prepare_raspbian_check_and_set_value_boolean {
  local -r var_name="$1"
  local -r var_value="$2"

  if prepare_raspbian_config_check_var_name "${var_name}" ; then
    _prepare_raspbian_config_set_value_ "${var_name}" "${var_value}"
  else
    return 1
  fi
}

function prepare_raspbian_check_and_set_value_integer {
  local -r var_name="$1"
  local -r var_value="$2"

  if prepare_raspbian_config_check_var_name "${var_name}" ; then
    _prepare_raspbian_config_set_value_ "${var_name}" "${var_value}"
  else
    return 1
  fi
}

function _prepare_raspbian_config_set_link_ {
  local -r var_name="$1"
  local -r var_link="$2"

  ensure_raspbian_config_ready

  _prepare_raspbian_config_json_set_ "${var_name}" 'link' "${var_link}"
}

function prepare_raspbian_check_and_set_value_link {
  local -r var_name="$1"
  local -r var_link="$2"

  if prepare_raspbian_config_check_var_name "${var_name}" ; then
    _prepare_raspbian_config_set_link_ "${var_name}" "${var_link}"
  else
    return 1
  fi
}

function prepare_raspbian_check_and_set_value {
  local -r var_type="$1"
  local -r var_name="$2"
  local -r var_value="$3"

  prepare_raspbian_config_check_var_name "${var_name}" || exit 1

  local var_value_expected_type=
  var_value_expected_type="$( prepare_raspbian_get_type "${var_name}" )" || exit 1

  if [ -z "${var_value_expected_type}" ] ; then
    echofatal "Don't know how to handle '${var_name}'."
    exit 1
  fi
  if [ "${var_type}" != "${var_value_expected_type}" ] ; then
    echofatal "'${var_name}' must be use type: '${var_value_expected_type}'."
    exit 1
  fi

  case "${var_type}" in
    'BOOLEAN')
      if ! is_boolean "${var_value}" ; then
        echofatal "Value must be 'true' or 'false': '${var_value}'"
        exit 1
      fi
      prepare_raspbian_check_and_set_value_boolean "${var_name}" "${var_value}"
      ;;

    'INTEGER')
      if ! is_positive_integer "${var_value}" ; then
        echofatal "Value must be a postive integer: '${var_value}'"
        exit 1
      fi
      prepare_raspbian_check_and_set_value_integer "${var_name}" "${var_value}"
      ;;

    'LINK')
      if [ ! -f "${var_value}" ] ; then
        echofatal "File not found: '${var_value}'"
        exit 1
      fi
      prepare_raspbian_check_and_set_value_link "${var_name}" "${var_value}"
      ;;

    'STRING')
      prepare_raspbian_check_and_set_value_string "${var_name}" "${var_value}"
      ;;

    *)
      echofatal "Unsupported variable type: '${var_type}'"
      exit 1
      ;;
  esac
}

function prepare_raspbian_check_and_get_value {
  local -r var_name="$1"

  if prepare_raspbian_config_check_var_name "${var_name}" ; then
    prepare_raspbian_initialise_if_needed "${var_name}"

    if is_variable_define_and_not_empty_safe "${var_name}" ; then
      local -r value="${!var_name}"

      jq "{ \"${var_name}\" : \"${value}\" }" <<<'{}' || exit 1
      return 0
    else
      echo "${var_name} not define." >&2
      return 1
    fi
  else
    return 1
  fi
}
